// This file was initially copied from
// https://github.com/sciancio/websearch and is released under GPL
// version 2 or later.

const GLib    = imports.gi.GLib;
const GObject = imports.gi.GObject;
const Gio     = imports.gi.Gio;
const Gtk     = imports.gi.Gtk;
const Me      = imports.misc.extensionUtils.getCurrentExtension();

const Settings = Me.imports.convenience.getSettings();

function init() {
}

function connect(actor, event, callback) {
    let hook = actor.connect(event, callback);
    return function() {
        actor.disconnect(hook);
    };
}

function connectAll(actor, events, callback) {
    let hooks = events.map(function(event, i, a) {
        return actor.connect(event, callback);
    });
    return function() {
        while(hooks.length > 0) {
            actor.disconnect(hooks.pop());
        }
    };
}

function getWorkaroundDelay() {
    return Settings.get_int('workaround-delay');
}

function getMenus() {
    let groups = Settings.get_value('menus');
    let menus  = [];
    
    for (let i = 0, n = groups.n_children(); i < n; ++i) {
        menus[i] = [];
        let group = groups.get_child_value(i);
        
        for (let j = 0, n = group.n_children(); j < n; ++j) {
            menus[i][j] = group.get_child_value(j).get_string().toString().split(',', 1)[0];
        }
    }
    return menus;
}

function getHiddenApps() {
    return Settings.get_strv('hidden-apps');
}

function getMovedApps() {
    return Settings.get_strv('moved-apps');
}

function onUserMenuChange(f) {
    return connect(Settings, 'changed::moved-apps', f);
}

function onAppsChange(f) {
    return connectAll(Settings, [ 'changed::menus',
                                  'changed::hidden-apps',
                                  'changed::moved-apps' ], f);
}

function onViewApps(f) {
    let noFlags = imports.gi.Meta.KeyBindingFlags.NONE;
    global.display.add_keybinding('apps', Settings, noFlags, f);

    return function() {
        global.display.remove_keybinding('apps');
    };
}

function onSearch(f){
    let noFlags = imports.gi.Meta.KeyBindingFlags.NONE;
    global.display.add_keybinding('search', Settings, noFlags, f);

    return function() {
        global.display.remove_keybinding('search');
    };
}

function getKeybindingsWidgets() {
    let bindings = {
        'apps': 'Show apps menu',
        'search': 'Search'
    };

    let model = new Gtk.ListStore();

    model.set_column_types([
        GObject.TYPE_STRING,
        GObject.TYPE_STRING,
        GObject.TYPE_INT,
        GObject.TYPE_INT
    ]);

    for (name in bindings) {
        let [key, mods] = Gtk.accelerator_parse(Settings.get_strv(name)[0]);
        let row = model.insert(10);
        model.set(row, [0, 1, 2, 3], [name, bindings[name], mods, key ]);
    }

    let treeview = new Gtk.TreeView({
        'expand': true,
        'model': model,
        'headers-visible': false
    });

    // Action column
    let cellrend = new Gtk.CellRendererText();
    let col = new Gtk.TreeViewColumn({ 'title': 'Action', 'expand': true });
    col.pack_start(cellrend, true);
    col.add_attribute(cellrend, 'text', 1);
    treeview.append_column(col);

    // keybinding column
    cellrend = new Gtk.CellRendererAccel({
        'editable': true,
        'accel-mode': Gtk.CellRendererAccelMode.GTK
    });

    connect(cellrend, 'accel-edited', function(rend, iter, key, mods) {
        let value = Gtk.accelerator_name(key, mods);
        let [succ, iterator ] = model.get_iter_from_string(iter);

        if(!succ) {
            throw new Error('Error updating Keybinding');
        }

        let name = model.get_value(iterator, 0);

        model.set(iterator, [ 2, 3], [ mods, key ]);
        Settings.set_strv(name, [value]);
        Gio.Settings.sync();
    });

    col = new Gtk.TreeViewColumn({ 'title': 'Shortcut', 'expand': true });

    col.pack_end(cellrend, false);
    col.add_attribute(cellrend, 'accel-mods', 2);
    col.add_attribute(cellrend, 'accel-key', 3);

    treeview.append_column(col);

	let box = new Gtk.Box({'vexpand' : false});
	box.add(treeview);

    let keysLabel = new Gtk.Label({
        label  : 'Keyboard shortcuts:',
        xalign : 0.0
    });
    return [keysLabel, box];
}

function getMenusWidgets() {
    function toString(menus) {
        return menus.map(function(menu, i, a) {
            return '[' + menu.join(', ') + ']';
        }).join(', ');
    }

    function toArray(str) {
        return str.replace(/\s*,\s*/g, ',').replace(/\s*\[\s*/g, '[').replace(/\s*\]\s*/g, ']')
            .replace(/^\[/g, '').replace(/\]$/g, '').split('],[').map(function(e, i, a) {
                return e.split(',');
            });
    }

    let label = new Gtk.Label({
        label  : 'Menu groups:',
        xalign : 0.0
    });
    let entry = new Gtk.Entry({
        text : toString(getMenus())
    });
    let button = new Gtk.Button({ 
        label : 'Apply'
    });
    
    connect(button, 'clicked', function() {
        try {
            let config = GLib.Variant.new('aas', toArray(entry.get_text()));
            Settings.set_value('menus', config);
            Gio.Settings.sync();
        } catch (e) {}
        entry.set_text(toString(getMenus()));
    });
    
    return [label, entry, button];
}

function getHiddenAppsWidgets() {
    let label = new Gtk.Label({
        label  : 'Hidden apps:',
        xalign : 0.0
    });
    let entry = new Gtk.Entry({
        text : getHiddenApps().join(', ')
    });
    let button = new Gtk.Button({ 
        label : 'Apply'
    });
    
    connect(button, 'clicked', function() {
        try {
            let config = GLib.Variant.new(
                'as',
                entry.get_text().replace(/\s*,\s*/g, ',').split(',')
            );
            Settings.set_value('hidden-apps', config);
            Gio.Settings.sync();
        } catch (e) {}
        entry.set_text(getHiddenApps().join(', '));
    });
    
    return [label, entry, button];
}

function getMovedAppsWidgets() {
    let label = new Gtk.Label({
        label  : 'Moved apps:',
        xalign : 0.0
    });
    let entry = new Gtk.Entry({
        text : getMovedApps().join(', ')
    });
    let button = new Gtk.Button({ 
        label : 'Apply'
    });
    
    connect(button, 'clicked', function() {
        try {
            let config = GLib.Variant.new(
                'as',
                entry.get_text().replace(/\s*,\s*/g, ',').split(',')
            );
            Settings.set_value('moved-apps', config);
            Gio.Settings.sync();
        } catch (e) {}
        entry.set_text(getMovedApps().join(', '));
    });
    
    return [label, entry, button];
}

function buildPrefsWidget() {
    let widget = new Gtk.Grid({
        margin_top   : 5,
        margin_left  : 5,
        margin_right : 5,
        row_spacing  : 5
    });
    
    let [keysLabel, keysBox] = getKeybindingsWidgets();
    let [menusLabel, menusEntry, menusButton] = getMenusWidgets();
    let [movedAppsLabel, movedAppsEntry, movedAppsButton] = getMovedAppsWidgets();
    let [hiddenAppsLabel, hiddenAppsEntry, hiddenAppsButton] = getHiddenAppsWidgets();

    // left   : the column number to attach the left side of child to
    // top    : the row number to attach the top side of child to
    // width  : the number of columns that child will span
    // height : the number of rows that child will span
    widget.attach(keysLabel,        0, 0, 1, 1);
    widget.attach(keysBox,          0, 1, 6, 2);
    widget.attach(menusLabel,       0, 3, 1, 1);
    widget.attach(menusEntry,       0, 4, 5, 1);
    widget.attach(menusButton,      5, 4, 1, 1);
    widget.attach(movedAppsLabel,   0, 5, 1, 1);
    widget.attach(movedAppsEntry,   0, 6, 5, 1);
    widget.attach(movedAppsButton,  5, 6, 1, 1);
    widget.attach(hiddenAppsLabel,  0, 7, 1, 1);
    widget.attach(hiddenAppsEntry,  0, 8, 5, 1);
    widget.attach(hiddenAppsButton, 5, 8, 1, 1);
    
    widget.show_all();
    return widget;
}
